
#ifdef HAS_GLOBUS_GSS

#include "auth.h"
#include "debug.h"

#include "globus_common.h"
#include "globus_gss_assist.h"

#include <stdio.h>
#include <unistd.h>
#include <string.h>

static int read_token(void *link, void **bufp, size_t *sizep)
{
	char line[AUTH_LINE_MAX];
	time_t stoptime = time(0)+60;
	int result;

	if(link_readline(link,line,sizeof(line),stoptime)) {
		*sizep = atoi(line);
		*bufp = malloc(*sizep);
		if(*bufp) {
			result = link_read(link,*bufp,*sizep,stoptime);
			if(result==*sizep) {
				return GLOBUS_SUCCESS;
			}
			free(*bufp);
		}
	}

	return GLOBUS_GSS_ASSIST_TOKEN_EOF;
}

static int write_token(void *link, void *buf, size_t size)
{
	char line[AUTH_LINE_MAX];
	time_t stoptime = time(0)+60;

	sprintf(line,"%d\n",size);
	link_write(link,line,strlen(line),stoptime);
	if(link_write(link,buf,size,stoptime)==size) {
		return GLOBUS_SUCCESS;
	} else {
		return GLOBUS_GSS_ASSIST_TOKEN_EOF;
	}
}

static int auth_globus_assert( struct link *link, time_t stoptime )
{
	gss_cred_id_t credential = GSS_C_NO_CREDENTIAL;
	gss_ctx_id_t context = GSS_C_NO_CONTEXT;
	OM_uint32 major, minor, flags=0;
	int token;
	int success = 0;

	debug("auth_globus: loading my credentials");
	major = globus_gss_assist_acquire_cred(&minor,GSS_C_INITIATE,&credential); 
	if( major==GSS_S_COMPLETE ) {
		debug("auth_globus: waiting for server to get ready");
		if(auth_barrier(link,"yes\n",stoptime)) {
			debug("auth_globus: authenticating with server");
			major = globus_gss_assist_init_sec_context( &minor, credential, &context, 0, 0, &flags, &token, read_token, link, write_token, link );
			if(major==GSS_S_COMPLETE) {
				debug("auth_globus: credentials accepted!");
				success = 1;
				gss_delete_sec_context( &minor, &context, GSS_C_NO_BUFFER );
			} else {
				debug("auth_globus: credentials rejected!");
			}
		} else {
			debug("auth_globus: server couldn't load credentials");
		}
		gss_release_cred( &major, &credential );
	} else {
		debug("auth_globus: couldn't load my credentials; did you grid-proxy-init?");
		auth_barrier(link,"no\n",stoptime);
	}

	return success;
}

static int auth_globus_accept( struct link *link, char **subject, time_t stoptime )
{
	gss_cred_id_t credential = GSS_C_NO_CREDENTIAL;
	gss_ctx_id_t context = GSS_C_NO_CONTEXT;
	OM_uint32 major, minor, flags=0;
	int token;
	int success = 0;

	*subject = 0;

	debug("auth_globus: loading my credentials");
	major = globus_gss_assist_acquire_cred(&minor,GSS_C_ACCEPT,&credential); 
	if(major==GSS_S_COMPLETE) {

		debug("auth_globus: waiting for client to get ready");
		if(auth_barrier(link,"yes\n",stoptime)) {

			debug("auth_globus: authenticating client");
			major = globus_gss_assist_accept_sec_context( &minor, &context, credential, subject, &flags, 0, &token, 0, read_token, link, write_token, link );
			if( major==GSS_S_COMPLETE ) {
				debug("auth_globus: accepted client %s",*subject);
				success = 1;
				gss_delete_sec_context( &minor, &context, GSS_C_NO_BUFFER );
			} else {
				debug("auth_globus: couldn't authenticate client");
			}
		} else {
			debug("auth_globus: client couldn't load credentials");
		}
		gss_release_cred( &major, &credential );
	} else {
		debug("auth_globus: couldn't load my credentials: did you run grid-proxy-init?");
		auth_barrier(link,"no\n",stoptime);
	}

	return success;
}

int auth_globus_register()
{
	debug("auth_globus: registered");
	return auth_register( "globus", auth_globus_assert, auth_globus_accept );
}

#else

#include "debug.h"

int auth_globus_register()
{
	debug("auth_globus: not compiled in");
	return 0;
}

#endif
