
#include "auth.h"
#include "link.h"
#include "getopt.h"
#include "debug.h"

#include "auth_kerberos.h"
#include "auth_anonymous.h"
#include "auth_filesystem.h"
#include "auth_hostname.h"
#include "auth_userpass.h"
#include "auth_globus.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

static void show_use( const char *cmd ) 
{
	fprintf(stderr,"Use: %s [options]\n",cmd);
	fprintf(stderr,"Where options are:\n");
	fprintf(stderr," -K        Allow Kerberos\n");
	fprintf(stderr," -G        Allow Globus\n");
	fprintf(stderr," -U        Allow userpass\n");
	fprintf(stderr," -H        Allow hostname\n");
	fprintf(stderr," -A        Allow anonymous\n");
	fprintf(stderr," -F        Allow filesystem\n");
	fprintf(stderr," -p <num>  Port number\n");
	fprintf(stderr," -h <name> Hostname\n");
	fprintf(stderr," -d        Debugging\n");
}

int main( int argc, char *argv[] )
{
	struct link *link, *master;
	char *subject=0, *host=0, *type=0, *luser=0;
	time_t stoptime;
	char line[1024];
	char c;
	int portnum=30000;
	char *hostname=0;
	int timeout=30;

	while((c=getopt(argc,argv,"AKGUHFp:h:d"))!=(char)-1) {
		switch(c) {
			case 'p':
				portnum = atoi(optarg);
				break;
			case 'h':
				hostname = optarg;
				break;
			case 'd':
				debug_config_console(1);
				break;
			case 'K':
				if(!auth_kerberos_register()) {
					fatal("couldn't register Kerberos authentication");
				}
				break;
			case 'G':
				if(!auth_globus_register()) {
					fatal("couldn't register Globus authentication");
				}
				break;
			case 'H':
				if(!auth_hostname_register()) {
					fatal("couldn't register hostname authentication");
				}
				break;
			case 'U':
				if(!auth_userpass_register("passwd")) {
					fatal("couldn't register userpass authentication");
				}
				break;
			case 'A':
				if(!auth_anonymous_register()) {
					fatal("couldn't register anonyous authentication");
				}
				break;
			case 'F':
                                if(!auth_filesystem_register("/tmp")) {
                                        fatal("couldn't register filesystem authentication");
                                }
                                break;                                                                
			default:
				show_use(argv[0]);
				exit(1);
		}
	}

	if(hostname) {
		stoptime = time(0)+timeout;

		link = link_connect(hostname,portnum,stoptime);
		if(!link) fatal("couldn't connect to %s:%d: %s",hostname,portnum,strerror(errno));

		if(auth_assert(link,&type,&subject,stoptime)) {
			printf("server thinks I am %s %s\n",type,subject);
			if(link_readline(link,line,sizeof(line),stoptime)) {
				printf("got message: %s\n",line);
			} else {
				printf("lost connection!\n");
			}
		} else {
			printf("unable to authenticate.\n");
		}

		link_close(link);

	} else {
		stoptime = time(0)+timeout;

		master = link_serve(portnum);
		if(!master) fatal("couldn't serve port %d: %s\n",portnum,strerror(errno));

		while(time(0)<stoptime) {
			link = link_accept(master,stoptime);
			if(!link) continue;

			if(auth_accept(link,"authfile",&type,&host,&subject,&luser,stoptime)) {
				time_t t = time(0);
				printf("accepted auth %s %s %s %s\n",type,host,subject,luser);
				sprintf(line,"Hello %s, it is now %s",subject,ctime(&t));
				link_write(link,line,strlen(line),stoptime);
				free(subject);
			} else {
				printf("couldn't auth accept\n");
			} 

			link_close(link);
		}
	}

	return 0;
}


