/*
dttools is Copyright (C) 2002 Douglas Thain
This software is released under the GNU General Public License.
See the file COPYING for details.
*/


#include "offset_buffer.h"
#include "debug.h"

#include <stdlib.h>

struct offset_buffer {
	unsigned headroom;
	unsigned headroom_history;
	unsigned length;
	unsigned markpos;
	char *data;
};

static unsigned max_headroom = 0;

struct offset_buffer * offset_buffer_create( unsigned size, unsigned with_headroom )
{
	struct offset_buffer *b;

	b = malloc(sizeof(*b));
	if(!b) return 0;

	if( with_headroom ) {
		b->headroom = max_headroom;
	} else {
		b->headroom = 0;
	}

	b->headroom_history = b->headroom;
	b->length = size + b->headroom;
	b->markpos = b->headroom;

	b->data = malloc(size+b->headroom);
	if(!b->data) {
		free(b);
		return 0;
	}

	b->data[0] = 0;

	return b;
}

void offset_buffer_delete( struct offset_buffer *b )
{
	free(b->data);
	free(b);
}

char * offset_buffer_data( struct offset_buffer *b )
{
	return &b->data[b->headroom];
}

unsigned offset_buffer_length( struct offset_buffer *b )
{
	return b->length-b->headroom;	
}

unsigned offset_buffer_prepend( struct offset_buffer *b, char *data, unsigned length )
{
	if( b->headroom>=length ) {
		b->headroom -= length;
		b->headroom_history += length;
		memcpy(&b->data[b->headroom],data,length);
		return 1;
	} else {
		unsigned newlength;
		char *newdata;

		newlength = b->length+length-b->headroom;
		newdata = malloc(newlength);
		if(!newdata) return 0;

		memcpy(newdata,data,length);
		memcpy(&newdata[length],&b->data[b->headroom],b->length-b->headroom);

		free(b->data);
		b->data = newdata;
		b->headroom = 0;
		b->length = newlength;
		b->headroom_history += length;
		b->markpos += length;

		if(b->headroom_history>max_headroom) {
			max_headroom = b->headroom_history;
			debug("offset_buffer_prepend: expanding max_headroom to %d",max_headroom);
		}
		return 1;
	}
}

unsigned offset_buffer_gets( struct offset_buffer *b, char *line, unsigned length )
{
	unsigned old_headroom = b->headroom;

	while(length>0) {
		if(length<=0) break;
		if(b->headroom>=b->length) break;

		*line = b->data[b->headroom];
		if(*line=='\n') {
			*line=0;
			b->headroom++;
			return 1;
		}

		length--;
		line++;
		b->headroom++;
	}

	b->headroom = old_headroom;
	return 0;
}

void offset_buffer_mark( struct offset_buffer *b )
{
	b->markpos = b->headroom;
}

void offset_buffer_reset( struct offset_buffer *b )
{
	b->headroom = b->markpos;
}
